@@ -34,7 +34,6 @@
 
 #define GET_DISTANCE(m, r, c) (r==c? 0 : (r < c ? m[c][r] : m[r][c]))
 
-using namespace std;
 using namespace boost;
 using namespace SpanningTreeClustering;
 
@@ -48,9 +47,9 @@ struct redcap_thread_args {
     Tree* tree;
     int start;
     int end;
-    vector<int> ids;
-    vector<pair<int, int> > od_array;
-    boost::unordered_map<int, vector<int> > nbr_dict;
+    std::vector<int> ids;
+    std::vector<std::pair<int, int> > od_array;
+    boost::unordered_map<int, std::vector<int> > nbr_dict;
 };
 
 void* redcap_thread_helper(void* voidArgs)
@@ -86,7 +85,7 @@ the pivot element at its correct position in sorted
 array, and places all smaller (smaller than pivot)
 to left of pivot and all greater elements to right
 of pivot */
-int partition (vector<Edge*>& arr, int low, int high)
+int partition (std::vector<Edge*>& arr, int low, int high)
 {
     Edge* pivot = arr[high]; // pivot
     int i = (low - 1); // Index of smaller element
@@ -115,7 +114,7 @@ int partition (vector<Edge*>& arr, int low, int high)
 arr[] --> Array to be sorted,
 low --> Starting index,
 high --> Ending index */
-void quickSort(vector<Edge*>& arr, int low, int high)
+void quickSort(std::vector<Edge*>& arr, int low, int high)
 {
     if (low < high)
     {
@@ -135,7 +134,7 @@ void quickSort(vector<Edge*>& arr, int low, int high)
 // SSDUtils
 //
 /////////////////////////////////////////////////////////////////////////
-void SSDUtils::MeasureSplit(double ssd, vector<int> &ids, int split_position,  Measure& result)
+void SSDUtils::MeasureSplit(double ssd, std::vector<int> &ids, int split_position,  Measure& result)
 {
     int start1 = 0;
     int end1 = split_position;
@@ -152,7 +151,7 @@ void SSDUtils::MeasureSplit(double ssd, vector<int> &ids, int split_position,  M
     result.ssd_part2 = ssd2;
 }
 
-double SSDUtils::ComputeSSD(vector<int> &visited_ids, int start, int end)
+double SSDUtils::ComputeSSD(std::vector<int> &visited_ids, int start, int end)
 {
     int size = end - start;
     double sum_squared = 0.0;
@@ -249,7 +248,7 @@ Edge::Edge(Node* a, Node* b, double _length)
 // Tree
 //
 /////////////////////////////////////////////////////////////////////////
-Tree::Tree(vector<int> _ordered_ids, vector<Edge*> _edges, AbstractClusterFactory* _cluster)
+Tree::Tree(std::vector<int> _ordered_ids, std::vector<Edge*> _edges, AbstractClusterFactory* _cluster)
   : cluster(_cluster),  edges(_edges), ordered_ids(_ordered_ids), cpu_threads(_cluster->cpu_threads)
 {
     ssd_reduce = 0;
@@ -273,7 +272,7 @@ Tree::Tree(vector<int> _ordered_ids, vector<Edge*> _edges, AbstractClusterFactor
         }
 
         // use edges and ordered_ids to create nbr_dict and od_array
-        boost::unordered_map<int, vector<int> > nbr_dict;
+        boost::unordered_map<int, std::vector<int> > nbr_dict;
         od_array.resize(edge_size);
 
         int o_id, d_id;
@@ -320,9 +319,9 @@ Tree::~Tree()
 {
 }
 
-void Tree::run_threads(vector<int>& ids,
-                       vector<pair<int, int> >& od_array,
-                       boost::unordered_map<int, vector<int> >& nbr_dict)
+void Tree::run_threads(std::vector<int>& ids,
+                       std::vector<std::pair<int, int> >& od_array,
+                       boost::unordered_map<int, std::vector<int> >& nbr_dict)
 {
     int nCPUs = cpu_threads;
     int n_jobs = od_array.size();
@@ -381,9 +380,9 @@ void Tree::run_threads(vector<int>& ids,
 #endif
 }
 
-void Tree::Partition(int start, int end, vector<int>& ids,
-                           vector<pair<int, int> >& od_array,
-                           boost::unordered_map<int, vector<int> >& nbr_dict)
+void Tree::Partition(int start, int end, std::vector<int>& ids,
+                           std::vector<std::pair<int, int> >& od_array,
+                           boost::unordered_map<int, std::vector<int> >& nbr_dict)
 {
     int size = (int)nbr_dict.size();
     int orig_id, dest_id;
@@ -394,7 +393,7 @@ void Tree::Partition(int start, int end, vector<int>& ids,
     int best_pos = -1;
     double tmp_ssd_reduce = 0, tmp_ssd=0;
 
-    vector<int> visited_ids(size), best_ids(size);
+    std::vector<int> visited_ids(size), best_ids(size);
 
     // cut edge one by one
     for ( i=start; i<=end; i++) {
@@ -402,7 +401,7 @@ void Tree::Partition(int start, int end, vector<int>& ids,
         dest_id = od_array[i].second;
 
         int idx = 0;
-        vector<int> cand_ids(max_id+1, -1);
+        std::vector<int> cand_ids(max_id+1, -1);
         Split(orig_id, dest_id, nbr_dict, cand_ids);
 
         for (size_t j=0; j<ids.size(); j++) {
@@ -458,9 +457,9 @@ void Tree::Partition(int start, int end, vector<int>& ids,
     }
 }
 
-void Tree::Split(int orig, int dest, boost::unordered_map<int, vector<int> >& nbr_dict, vector<int>& cand_ids)
+void Tree::Split(int orig, int dest, boost::unordered_map<int, std::vector<int> >& nbr_dict, std::vector<int>& cand_ids)
 {
-    stack<int> visited_ids;
+    std::stack<int> visited_ids;
     int cur_id, i, nbr_size, nbr;
 
     visited_ids.push(orig);
@@ -468,7 +467,7 @@ void Tree::Split(int orig, int dest, boost::unordered_map<int, vector<int> >& nb
         cur_id = visited_ids.top();
         visited_ids.pop();
         cand_ids[cur_id] = 1;
-        vector<int>& nbrs = nbr_dict[cur_id];
+        std::vector<int>& nbrs = nbr_dict[cur_id];
         nbr_size = (int)nbrs.size();
         for (i=0; i<nbr_size; i++) {
             nbr = nbrs[i];
@@ -479,7 +478,7 @@ void Tree::Split(int orig, int dest, boost::unordered_map<int, vector<int> >& nb
     }
 }
 
-bool Tree::checkControl(vector<int>& cand_ids, vector<int>& ids, int flag)
+bool Tree::checkControl(std::vector<int>& cand_ids, std::vector<int>& ids, int flag)
 {
     if (controls == NULL) {
         return true;
@@ -495,14 +494,14 @@ bool Tree::checkControl(vector<int>& cand_ids, vector<int>& ids, int flag)
     return val >= control_thres;
 }
 
-pair<Tree*, Tree*> Tree::GetSubTrees()
+std::pair<Tree*, Tree*> Tree::GetSubTrees()
 {
     if (split_ids.empty()) {
         return this->subtrees;
     }
     int size = (int)this->split_ids.size();
-    vector<int> part1_ids(this->split_pos);
-    vector<int> part2_ids(size -this->split_pos);
+    std::vector<int> part1_ids(this->split_pos);
+    std::vector<int> part2_ids(size -this->split_pos);
 
     int max_id = -1;
     for (int i=0; i<size; i++) {
@@ -516,10 +515,10 @@ pair<Tree*, Tree*> Tree::GetSubTrees()
         }
     }
 
-    vector<Edge*> part1_edges(part1_ids.size()-1);
-    vector<Edge*> part2_edges(part2_ids.size()-1);
+    std::vector<Edge*> part1_edges(part1_ids.size()-1);
+    std::vector<Edge*> part2_edges(part2_ids.size()-1);
 
-    vector<int> part_index(max_id+1, 0);
+    std::vector<int> part_index(max_id+1, 0);
     for (size_t i=0; i< part1_ids.size(); i++) {
         part_index[ part1_ids[i] ] = -1;
     }
@@ -554,7 +553,7 @@ pair<Tree*, Tree*> Tree::GetSubTrees()
 // AbstractClusterFactory
 //
 ////////////////////////////////////////////////////////////////////////////////
-AbstractClusterFactory::AbstractClusterFactory(int row, int col,  double** _distances, double** _data, const vector<bool>& _undefs, GeoDaWeight * _w, int cpu_threads)
+AbstractClusterFactory::AbstractClusterFactory(int row, int col,  double** _distances, double** _data, const std::vector<bool>& _undefs, GeoDaWeight * _w, int cpu_threads)
   : rows(row), cols(col),w(_w), dist_matrix(_distances), raw_data(_data), undefs(_undefs), cpu_threads(cpu_threads)
 {
 }
@@ -589,20 +588,20 @@ void AbstractClusterFactory::init()
     Node* orig;
     Node* dest;
     double length;
-    boost::unordered_map<pair<int, int>, bool> access_dict;
+    boost::unordered_map<std::pair<int, int>, bool> access_dict;
 
     for (int i=0; i<rows; i++) {
         orig = nodes[i];
-        const vector<long>& nbrs = w->GetNeighbors(i);
+        const std::vector<long>& nbrs = w->GetNeighbors(i);
         for (int j=0; j<nbrs.size(); j++) {
             int nbr = (int)nbrs[j];
             dest = nodes[nbr];
             length = GET_DISTANCE(dist_matrix, orig->id, dest->id);
 
-            if (access_dict.find(make_pair(i, nbr)) == access_dict.end()) {
+            if (access_dict.find(std::make_pair(i, nbr)) == access_dict.end()) {
                 edges.push_back(new Edge(orig, dest, length));
-                access_dict[make_pair(i, nbr)] = true;
-                access_dict[make_pair(nbr, i)] = true;
+                access_dict[std::make_pair(i, nbr)] = true;
+                access_dict[std::make_pair(nbr, i)] = true;
             }
             this->dist_dict[i][nbr] = length;
         }
@@ -612,14 +611,14 @@ void AbstractClusterFactory::init()
 
 }
 
-vector<vector<int> >& AbstractClusterFactory::GetRegions()
+std::vector<std::vector<int> >& AbstractClusterFactory::GetRegions()
 {
     return cluster_ids;
 }
 
 void AbstractClusterFactory::Partitioning(int k)
 {
-    vector<Tree*> not_split_trees;
+    std::vector<Tree*> not_split_trees;
     Tree* current_tree = new Tree(ordered_ids, ordered_edges, this);
     PriorityQueue sub_trees;
     sub_trees.push(current_tree);
@@ -636,7 +635,7 @@ void AbstractClusterFactory::Partitioning(int k)
             continue;
         }
 
-        pair<Tree*, Tree*> children = tmp_tree->GetSubTrees();
+        std::pair<Tree*, Tree*> children = tmp_tree->GetSubTrees();
 
         Tree* left_tree = children.first;
         Tree* right_tree = children.second;
@@ -681,7 +680,7 @@ void AbstractClusterFactory::Partitioning(int k)
 // Skater
 //
 ////////////////////////////////////////////////////////////////////////////////
-Skater::Skater(int rows, int cols, double** _distances, double** _data, const vector<bool>& _undefs, GeoDaWeight* w, double* _controls, double _control_thres, int cpu_threads)
+Skater::Skater(int rows, int cols, double** _distances, double** _data, const std::vector<bool>& _undefs, GeoDaWeight* w, double* _controls, double _control_thres, int cpu_threads)
 : AbstractClusterFactory(rows, cols, _distances, _data, _undefs, w, cpu_threads)
 {
     controls = _controls;
@@ -697,24 +696,24 @@ Skater::~Skater()
 void Skater::Clustering()
 {
     Graph g(rows);
-    boost::unordered_map<pair<int, int>, bool> access_dict;
+    boost::unordered_map<std::pair<int, int>, bool> access_dict;
     for (int i=0; i<rows; i++) {
         const std::vector<long>& nbrs = w->GetNeighbors(i);
         for (int j=0; j<nbrs.size(); j++) {
-            if (access_dict.find(make_pair(i, nbrs[j])) == access_dict.end()) {
+            if (access_dict.find(std::make_pair(i, nbrs[j])) == access_dict.end()) {
                 boost::add_edge(i, nbrs[j], GET_DISTANCE(dist_matrix, i, nbrs[j]), g);
-                access_dict[make_pair(i, nbrs[j])] = true;
-                access_dict[make_pair(nbrs[j], i)] = true;
+                access_dict[std::make_pair(i, nbrs[j])] = true;
+                access_dict[std::make_pair(nbrs[j], i)] = true;
             }
         }
     }
 
     boost::unordered_map<int, bool> id_dict;
-    boost::unordered_map<pair<int, int>, Edge*> edge_dict;
+    boost::unordered_map<std::pair<int, int>, Edge*> edge_dict;
     for (size_t i=0; i<edges.size(); i++) {
         Edge* e = edges[i];
-        edge_dict[make_pair(e->orig->id, e->dest->id)] = e;
-        edge_dict[make_pair(e->dest->id, e->orig->id)] = e;
+        edge_dict[std::make_pair(e->orig->id, e->dest->id)] = e;
+        edge_dict[std::make_pair(e->dest->id, e->orig->id)] = e;
     }
 
     //https://github.com/vinecopulib/vinecopulib/issues/22
@@ -728,8 +727,8 @@ void Skater::Clustering()
         if ((int)source != target) {
             //boost::add_edge(source, p[source], mst);
             //ordered_edges.push_back(new Edge(source, p[source]));
-            pair<int,int> o_d(source, target);
-            pair<int,int> d_o(target, source);
+            std::pair<int,int> o_d(source, target);
+            std::pair<int,int> d_o(target, source);
             if (edge_dict.find(o_d) != edge_dict.end() &&
                 edge_dict.find(d_o) != edge_dict.end())
             {
@@ -763,7 +762,7 @@ void Skater::Clustering()
 // 1 FirstOrderSLKRedCap
 //
 ////////////////////////////////////////////////////////////////////////////////
-FirstOrderSLKRedCap::FirstOrderSLKRedCap(int rows, int cols, double** _distances, double** _data, const vector<bool>& _undefs, GeoDaWeight* w, double* _controls, double _control_thres, int cpu_threads)
+FirstOrderSLKRedCap::FirstOrderSLKRedCap(int rows, int cols, double** _distances, double** _data, const std::vector<bool>& _undefs, GeoDaWeight* w, double* _controls, double _control_thres, int cpu_threads)
 : AbstractClusterFactory(rows, cols, _distances, _data, _undefs, w, cpu_threads)
 {
     controls = _controls;
@@ -839,7 +838,7 @@ void FirstOrderSLKRedCap::Clustering()
 // each merge.
 //
 ////////////////////////////////////////////////////////////////////////////////
-FirstOrderALKRedCap::FirstOrderALKRedCap(int rows, int cols, double** _distances, double** _data, const vector<bool>& _undefs, GeoDaWeight * w, double* _controls, double _control_thres, int cpu_threads)
+FirstOrderALKRedCap::FirstOrderALKRedCap(int rows, int cols, double** _distances, double** _data, const std::vector<bool>& _undefs, GeoDaWeight * w, double* _controls, double _control_thres, int cpu_threads)
 : AbstractClusterFactory(rows, cols, _distances, _data, _undefs, w, cpu_threads)
 {
     controls = _controls;
@@ -863,7 +862,7 @@ void FirstOrderALKRedCap::Clustering()
 // 3 FirstOrderCLKRedCap
 //
 ////////////////////////////////////////////////////////////////////////////////
-FirstOrderCLKRedCap::FirstOrderCLKRedCap(int rows, int cols, double** _distances, double** _data, const vector<bool>& _undefs, GeoDaWeight * w, double* _controls, double _control_thres, int cpu_threads)
+FirstOrderCLKRedCap::FirstOrderCLKRedCap(int rows, int cols, double** _distances, double** _data, const std::vector<bool>& _undefs, GeoDaWeight * w, double* _controls, double _control_thres, int cpu_threads)
 : AbstractClusterFactory(rows, cols, _distances, _data, _undefs, w, cpu_threads)
 {
     controls = _controls;
@@ -886,7 +885,7 @@ void FirstOrderCLKRedCap::Clustering()
 // 4 FullOrderSLKRedCap
 //
 ////////////////////////////////////////////////////////////////////////////////
-FullOrderSLKRedCap::FullOrderSLKRedCap(int rows, int cols, double** _distances, double** _data, const vector<bool>& _undefs, GeoDaWeight * w, double* _controls, double _control_thres, int cpu_threads)
+FullOrderSLKRedCap::FullOrderSLKRedCap(int rows, int cols, double** _distances, double** _data, const std::vector<bool>& _undefs, GeoDaWeight * w, double* _controls, double _control_thres, int cpu_threads)
 : FullOrderALKRedCap(rows, cols, _distances, _data, _undefs, w, _controls, _control_thres, false, cpu_threads)
 {
     init();
@@ -897,7 +896,7 @@ FullOrderSLKRedCap::~FullOrderSLKRedCap()
 
 }
 
-double FullOrderSLKRedCap::UpdateClusterDist(int cur_id, int o_id, int d_id, bool conn_c_o, bool conn_c_d, vector<int>& clst_ids, vector<int>& clst_startpos, vector<int>& clst_nodenum)
+double FullOrderSLKRedCap::UpdateClusterDist(int cur_id, int o_id, int d_id, bool conn_c_o, bool conn_c_d, std::vector<int>& clst_ids, std::vector<int>& clst_startpos, std::vector<int>& clst_nodenum)
 {
     double new_dist = 0;
     if (conn_c_o && conn_c_d) {
@@ -932,7 +931,7 @@ double FullOrderSLKRedCap::UpdateClusterDist(int cur_id, int o_id, int d_id, boo
 // 5 FullOrderALKRedCap
 //
 ////////////////////////////////////////////////////////////////////////////////
-FullOrderALKRedCap::FullOrderALKRedCap(int rows, int cols, double** _distances, double** _data, const vector<bool>& _undefs,  GeoDaWeight * w, double* _controls, double _control_thres, bool init_flag, int cpu_threads)
+FullOrderALKRedCap::FullOrderALKRedCap(int rows, int cols, double** _distances, double** _data, const std::vector<bool>& _undefs,  GeoDaWeight * w, double* _controls, double _control_thres, bool init_flag, int cpu_threads)
 : AbstractClusterFactory(rows, cols, _distances, _data, _undefs, w, cpu_threads)
 {
     controls = _controls;
@@ -950,7 +949,7 @@ FullOrderALKRedCap::~FullOrderALKRedCap()
 void FullOrderALKRedCap::Clustering()
 {
     int num_nodes = (int)nodes.size();
-    vector<Node*> ordered_nodes(num_nodes);
+    std::vector<Node*> ordered_nodes(num_nodes);
 
     for (size_t i=0; i< this->edges.size(); i++) {
         Edge* edge = this->edges[i];
@@ -964,7 +963,7 @@ void FullOrderALKRedCap::Clustering()
     quickSort(edges, 0, edges.size()-1);
 
     int num_edges = (int)edges.size();
-    vector<Edge*> edges_copy(num_edges);
+    std::vector<Edge*> edges_copy(num_edges);
     for (int i=0; i<num_edges; i++) {
         edges_copy[i] = edges[i];
     }
@@ -973,13 +972,13 @@ void FullOrderALKRedCap::Clustering()
 
     this->ordered_edges.resize(num_nodes-1);
 
-    vector<int> ids(num_nodes);
+    std::vector<int> ids(num_nodes);
     // number of nodes in a cluster
     // the start position of a cluster
     // the cluster id of each nodes
-    vector<int> cluster_nodenum(num_nodes);
-    vector<int> cluster_ids(num_nodes);
-    vector<int> cluster_startpos(num_nodes);
+    std::vector<int> cluster_nodenum(num_nodes);
+    std::vector<int> cluster_ids(num_nodes);
+    std::vector<int> cluster_startpos(num_nodes);
     for (int i=0; i<num_nodes; ++i) {
         ids[i] = i;
         cluster_ids[i] = i;
@@ -990,9 +989,9 @@ void FullOrderALKRedCap::Clustering()
     int index = 0;
     int cnt = 0;
 
-    vector<bool> access_flag(num_nodes, false);
-    vector<int> counts(num_nodes);
-    vector<Edge*> new_edges;
+    std::vector<bool> access_flag(num_nodes, false);
+    std::vector<int> counts(num_nodes);
+    std::vector<Edge*> new_edges;
 
     Edge* cur_edge = edges_copy[0];
     for (int k=0; k<num_edges; k++) {
@@ -1120,7 +1119,7 @@ void FullOrderALKRedCap::Clustering()
     }
 }
 
-double FullOrderALKRedCap::UpdateClusterDist(int cur_id, int o_id, int d_id, bool conn_c_o, bool conn_c_d, vector<int>& clst_ids, vector<int>& clst_startpos, vector<int>& clst_nodenum)
+double FullOrderALKRedCap::UpdateClusterDist(int cur_id, int o_id, int d_id, bool conn_c_o, bool conn_c_d, std::vector<int>& clst_ids, std::vector<int>& clst_startpos, std::vector<int>& clst_nodenum)
 {
     double new_dist = 0;
     if (conn_c_o && conn_c_d) {
@@ -1154,7 +1153,7 @@ double FullOrderALKRedCap::UpdateClusterDist(int cur_id, int o_id, int d_id, boo
     return new_dist;
 }
 
-Edge* FullOrderALKRedCap::GetShortestEdge(vector<Edge*>& _edges, int start, int end)
+Edge* FullOrderALKRedCap::GetShortestEdge(std::vector<Edge*>& _edges, int start, int end)
 {
     double len = DBL_MAX;
     Edge* short_e = NULL;
@@ -1171,7 +1170,7 @@ Edge* FullOrderALKRedCap::GetShortestEdge(vector<Edge*>& _edges, int start, int
 // 6 FullOrderCLKRedCap
 //
 ////////////////////////////////////////////////////////////////////////////////
-FullOrderCLKRedCap::FullOrderCLKRedCap(int rows, int cols, double** _distances, double** _data, const vector<bool>& _undefs, GeoDaWeight * w, double* _controls, double _control_thres, int cpu_threads)
+FullOrderCLKRedCap::FullOrderCLKRedCap(int rows, int cols, double** _distances, double** _data, const std::vector<bool>& _undefs, GeoDaWeight * w, double* _controls, double _control_thres, int cpu_threads)
 : FullOrderALKRedCap(rows, cols, _distances, _data, _undefs, w, _controls, _control_thres, false, cpu_threads)
 {
     init();
@@ -1182,7 +1181,7 @@ FullOrderCLKRedCap::~FullOrderCLKRedCap()
 
 }
 
-double FullOrderCLKRedCap::UpdateClusterDist(int cur_id, int o_id, int d_id, bool conn_c_o, bool conn_c_d, vector<int>& clst_ids, vector<int>& clst_startpos, vector<int>& clst_nodenum)
+double FullOrderCLKRedCap::UpdateClusterDist(int cur_id, int o_id, int d_id, bool conn_c_o, bool conn_c_d, std::vector<int>& clst_ids, std::vector<int>& clst_startpos, std::vector<int>& clst_nodenum)
 {
     double new_dist = 0.0;
     if (conn_c_o && conn_c_d) { // cur_id connects to both o and d now
@@ -1218,7 +1217,7 @@ double FullOrderCLKRedCap::UpdateClusterDist(int cur_id, int o_id, int d_id, boo
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-FullOrderWardRedCap::FullOrderWardRedCap(int rows, int cols, double** _distances, double** _data, const vector<bool>& _undefs,  GeoDaWeight * w, double* _controls, double _control_thres, int cpu_threads)
+FullOrderWardRedCap::FullOrderWardRedCap(int rows, int cols, double** _distances, double** _data, const std::vector<bool>& _undefs,  GeoDaWeight * w, double* _controls, double _control_thres, int cpu_threads)
 : FullOrderALKRedCap(rows, cols, _distances, _data, _undefs, w, _controls, _control_thres, false, cpu_threads)
 {
     init();
@@ -1232,7 +1231,7 @@ FullOrderWardRedCap::~FullOrderWardRedCap()
 void FullOrderWardRedCap::Clustering()
 {
     int num_nodes = (int)nodes.size();
-    vector<Node*> ordered_nodes(num_nodes);
+    std::vector<Node*> ordered_nodes(num_nodes);
 
     for (size_t i=0; i< this->edges.size(); i++) {
         Edge* edge = this->edges[i];
@@ -1246,7 +1245,7 @@ void FullOrderWardRedCap::Clustering()
     quickSort(edges, 0, edges.size()-1);
 
     int num_edges = (int)edges.size();
-    vector<Edge*> edges_copy(num_edges);
+    std::vector<Edge*> edges_copy(num_edges);
     for (int i=0; i<num_edges; i++) {
         edges_copy[i] = edges[i];
     }
@@ -1255,13 +1254,13 @@ void FullOrderWardRedCap::Clustering()
 
     this->ordered_edges.resize(num_nodes-1);
 
-    vector<int> ids(num_nodes);
+    std::vector<int> ids(num_nodes);
     // number of nodes in a cluster
     // the start position of a cluster
     // the cluster id of each nodes
-    vector<int> cluster_nodenum(num_nodes);
-    vector<int> cluster_ids(num_nodes);
-    vector<int> cluster_startpos(num_nodes);
+    std::vector<int> cluster_nodenum(num_nodes);
+    std::vector<int> cluster_ids(num_nodes);
+    std::vector<int> cluster_startpos(num_nodes);
     for (int i=0; i<num_nodes; ++i) {
         ids[i] = i;
         cluster_ids[i] = i;
@@ -1272,9 +1271,9 @@ void FullOrderWardRedCap::Clustering()
     int index = 0;
     int cnt = 0;
 
-    vector<bool> access_flag(num_nodes, false);
-    vector<int> counts(num_nodes);
-    vector<Edge*> new_edges;
+    std::vector<bool> access_flag(num_nodes, false);
+    std::vector<int> counts(num_nodes);
+    std::vector<Edge*> new_edges;
 
     Edge* cur_edge = edges_copy[0];
     for (int k=0; k<num_edges; k++) {
